//: /be/optiloading/DrawSF.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.util.ArrayList;

/** 
 * DrawSF class is used to draw the aproximate Shear Force diagram.
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank" href="http://www.optiloading.be">www.optiloading.be</a>
 */

public class DrawSF extends JPanel{
	private ArrayList<Integer> x;
	private ArrayList<Integer> y;
	
	/**
	 * Constructor
	 */
	public DrawSF(){
		Dimension d5 = new Dimension(599,300);
		setSize(d5);
		newList();
		addList(584,150);
	}
	
	/**
	 * Overwritten paintComponent method to draw the GZ-curve on the panel.
	 * @param g Graphics object containing what to render.
	 */
	public void paintComponent(Graphics g){
		Graphics2D g2d = (Graphics2D)g;
		
		RenderingHints rh = new RenderingHints(
				RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		g2d.setRenderingHints(rh);
		
		g2d.setColor(new Color(115,183,253)); //back color
		g2d.fillRect(0,0,599,300);
		
		g2d.setColor(Color.black);
		g2d.drawLine(27,150,594,150); //x-axis
		
		Stroke defaultStroke = g2d.getStroke();
		float[] dashPattern = { 3, 3, 3, 3};
		g2d.setStroke(new BasicStroke(1, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 10, dashPattern, 0));
		g2d.drawLine(37,0,37,300); //AP mark
		g2d.drawLine(584,0,584,300); //FP mark
		g2d.drawLine(131,0,131,300); //Frame 46
		g2d.drawLine(203,0,203,300); //Frame 78
		g2d.drawLine(275,0,275,300); //Frame 110
		g2d.drawLine(347,0,347,300); //Frame 142
		g2d.drawLine(419,0,419,300); //Frame 174
		g2d.drawLine(491,0,491,300); //Frame 206
		g2d.drawLine(565,0,565,300); //Frame 239
		g2d.drawLine(27,114,594,114); //5000 mark
		g2d.drawLine(27,79,594,79); // 10000 mark
		g2d.drawLine(27,43,594,43); // 15000 mark
		g2d.drawLine(27,7,594,7); // 20000 mark
		g2d.drawLine(27,185,594,185); //-5000 mark
		g2d.drawLine(27,221,594,221); // -10000 mark
		g2d.drawLine(27,257,594,257); // -15000 mark
		g2d.drawLine(27,293,594,293); // -20000 mark
		Font font = new Font("Dialog", Font.BOLD, 12);
		g2d.setFont(font);
		g2d.drawString("kton",3,14);
		font = new Font("Dialog", Font.PLAIN, 10);
		g2d.setFont(font);
		g2d.drawString("0",12,154);
		g2d.drawString("5",12,118);
		g2d.drawString("10",12,83);
		g2d.drawString("15",12,47);
		g2d.drawString("-5",11,189);
		g2d.drawString("-10",5,225);
		g2d.drawString("-15",5,261);
		g2d.drawString("-20",5,297);
		font = new Font("Dialog", Font.PLAIN, 10);
		g2d.setFont(font);
		g2d.drawString("AP",23,161);
		g2d.drawString("FP",586,161);
		g2d.drawString("Tank 1",514,292);
		g2d.drawString("Tank 2",439,292);
		g2d.drawString("Tank 3",367,292);
		g2d.drawString("Tank 4",295,292);
		g2d.drawString("Tank 5",223,292);
		g2d.drawString("Tank 6",151,292);
		
		//Still water limit
		g2d.setColor(Color.yellow);
		g2d.setStroke(defaultStroke);
		g2d.drawLine(37,150,131,64);
		g2d.drawLine(131,64,311,48);
		g2d.drawLine(311,48,491,59);
		g2d.drawLine(491,59,564,47);
		g2d.drawLine(564,47,584,150);
		g2d.drawLine(37,150,131,236);
		g2d.drawLine(131,236,311,252);
		g2d.drawLine(311,252,491,241);
		g2d.drawLine(491,241,564,253);
		g2d.drawLine(564,253,584,150);
		
		//Wave limit
		g2d.setColor(Color.red);
		g2d.drawLine(37,150,131,79);
		g2d.drawLine(131,79,311,64);
		g2d.drawLine(311,64,491,79);
		g2d.drawLine(491,79,564,47);
		g2d.drawLine(564,47,584,150);
		g2d.drawLine(37,150,131,221);
		g2d.drawLine(131,221,311,236);
		g2d.drawLine(311,236,491,221);
		g2d.drawLine(491,221,564,253);
		g2d.drawLine(564,253,584,150);
		
		//Real
		g2d.setColor(Color.white);
		GeneralPath graph = new GeneralPath();
		graph.moveTo(37,150);
		//The line is made up from the 2 coordinate lists
		for(int i = 0; i<x.size(); i++){
			graph.lineTo(x.get(i),y.get(i));
		}
		g2d.draw(graph);
	}
	
	/**
	 * Initiate a new list
	 */
	public void newList(){
		x = new ArrayList<Integer>();
		y = new ArrayList<Integer>();
	}
	
	/**
	 * Add x and y coordinates to the list
	 * @param xvalue Integer indicating the x coordinate.
	 * @param yvalue Integer indicating the y coordinate.
	 */
	public void addList(int xvalue, int yvalue){
		x.add(xvalue);
		y.add(yvalue);
	}
}