//: /be/optiloading/Gui.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.text.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.border.*;
import java.util.*;
import java.net.*;
import java.io.*;
import javax.imageio.*;

/** 
 * Gui class creates the main GUI.
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank" href="http://www.optiloading.be">www.optiloading.be</a>
 */

public class Gui{
	//Gui components that need to be accessed by methods
	private JFrame frame;
	private JButton start;
	private JFormattedTextField inputcargo;
	private JSpinner specgrav;
	private JSpinner waterdens;
	private JFormattedTextField inputtrim;
	private JFormattedTextField specgravfield;
	private JFormattedTextField waterdensfield;
	private JLabel besttrim;
	private JLabel draughtaft;
	private JLabel draughtfwd;
	private JLabel tmd;
	private JLabel tank1;
	private JLabel tank2;
	private JLabel tank3;
	private JLabel tank4;
	private JLabel tank5;
	private JLabel tank6;
	private JLabel displacement;
	private JLabel deadweight;
	private JLabel cargovolume;
	private JLabel gm;
	private JLabel fsc;
	private JLabel gm2;
	private JLabel fsc2;
	private JLabel dynstab;
	private JLabel generation;
	private JLabel bestfitness;
	private JLabel meanfitness;
	private JLabel maxbm;
	private JLabel maxsf;
	private JComboBox tmdorcargo;
	private JLayeredPane sfandbm;
	private JTextArea log;
	private DrawSF sfdrawing;
	private DrawBM bmdrawing;
	private DrawShip shipdrawing;
	private DrawGZ gzpanel;
	private RotatePanel shiprotated;
	private Chromosome bestchrom;
	
	public MiscComp misccompartments;
	
	private static Gui uniqueInstance; // Singelton
	
	// Private constructor to only create one Gui object.
	private Gui(){
		buildGui();
	}
	
	/**
	 * Returns the uniqueInstance of the Gui object.
	 * @return Gui object.
	 */
	public static Gui getInstance(){
		
		//Create a new Gui object when no Gui object is in existance.
		if (uniqueInstance == null){
			uniqueInstance = new Gui();
		}
		return uniqueInstance;
	}
	
	// Method to build the whole GUI (add all the widgets).
	private void buildGui(){
		// Input Panel
		JPanel input = new JPanel();
		input.setLayout(new BoxLayout(input,BoxLayout.X_AXIS));
		input.setBorder(new CompoundBorder(
						new TitledBorder(new EtchedBorder(),"Input"),
						new EmptyBorder(5,5,0,5)));

		//Labels for the input fields
		JPanel labelpanel = new JPanel();
		labelpanel.setLayout(new GridLayout(4,1));
		String[] tmdorcargolist = {"To load (ton):","True mean draught (m):"};
		tmdorcargo = new JComboBox(tmdorcargolist);
		tmdorcargo.setBorder(new EmptyBorder(0,0,0,7));
		tmdorcargo.setToolTipText("Choose between a desired tonnage or a desired true mean draught");
		JLabel specificgravity = new JLabel(" Specific gravity:");
		JLabel desiredtrim  = new JLabel(" Desired trim (m):");
		JLabel waterdensity = new JLabel(" Water density:");

		//Input fields
		JPanel fieldpanel = new JPanel();
		fieldpanel.setLayout(new GridLayout(4,1));
		inputcargo = new JFormattedTextField(NumberFormat.getInstance());
		inputcargo.setHorizontalAlignment(JTextField.RIGHT);
		inputcargo.addFocusListener(new FieldListener());
		inputcargo.setToolTipText("Input the weight of the cargo or a desired TMD");
		SpinnerNumberModel specgravmod = new SpinnerNumberModel(1.000,1.000-0.501,1.000+0.025,0.001);
		specgrav = new JSpinner(specgravmod);
		specgrav.setToolTipText("Input the specific gravity of the cargo");
		specgrav.setEditor(new JSpinner.NumberEditor(specgrav, "0.000"));
		specgravfield = ((JSpinner.DefaultEditor) specgrav.getEditor()).getTextField();
		specgravfield.setFont(new Font("Dialog",Font.PLAIN,12));
		specgravfield.addFocusListener(new FieldListener());
		inputtrim = new JFormattedTextField(NumberFormat.getInstance());
		inputtrim.setHorizontalAlignment(JTextField.RIGHT);
		inputtrim.setToolTipText("Input the desired trim");
		SpinnerNumberModel specgravmod2 = new SpinnerNumberModel(1.025,1.000-0.001,1.000+0.025,0.001);
		waterdens = new JSpinner(specgravmod2);
		waterdens.setToolTipText("Input the density of the water in which the ship is floating");
		waterdens.setEditor(new JSpinner.NumberEditor(waterdens, "0.000"));
		waterdensfield = ((JSpinner.DefaultEditor) waterdens.getEditor()).getTextField();
		waterdensfield.setFont(new Font("Dialog",Font.PLAIN,12));
		waterdensfield.addFocusListener(new FieldListener());
		
		//Add labels to labelpanel and fields to fieldpanel
		labelpanel.add(tmdorcargo);
		fieldpanel.add(inputcargo);
		labelpanel.add(specificgravity);
		fieldpanel.add(specgrav);
		labelpanel.add(desiredtrim);
		fieldpanel.add(inputtrim);
		labelpanel.add(waterdensity);
		fieldpanel.add(waterdens);
		
		//Add labelpanel and fielpanel to the input panel
		input.add(labelpanel);
		input.add(fieldpanel);
		
		//Input+start
		JPanel inputandstart = new JPanel();
		inputandstart.setLayout(new BorderLayout());
		inputandstart.add(BorderLayout.CENTER,input);
		start = new JButton("Start");
		start.setToolTipText("To start the search algorithm click here");
		start.setVerticalTextPosition(AbstractButton.CENTER);
		start.addActionListener(new StartButtonListener());
		JPanel startpanel = new JPanel();
		startpanel.setLayout(new BorderLayout());
		startpanel.setBorder(new EmptyBorder(0,3,0,3));
		startpanel.add(BorderLayout.CENTER, start);
		inputandstart.add(BorderLayout.SOUTH,startpanel);
		
		//Miscellaneous compartments
		misccompartments = MiscComp.getInstance();
		
		//Input + Misc Compartments
		JPanel inputandmisc = new JPanel();
		inputandmisc.setLayout(new BoxLayout(inputandmisc, BoxLayout.X_AXIS));
		inputandmisc.setBorder(new EmptyBorder(5,5,5,5));
		inputandmisc.add(inputandstart);
		inputandmisc.add(misccompartments);
		
		//GZ curve
		JPanel gzmainpanel = new JPanel();
		gzmainpanel.setBorder(new EtchedBorder());
		gzpanel = new DrawGZ();
		gzmainpanel.add(gzpanel);
		
		//Stability output
		JPanel stabilityoutput = new JPanel();
		stabilityoutput.setLayout(new BoxLayout(stabilityoutput, BoxLayout.Y_AXIS));
		stabilityoutput.setBorder(new EmptyBorder(3,5,3,5));
		gm = new JLabel("GM: ....... m");
		fsc = new JLabel ("FSC: ....... m");
		dynstab = new JLabel ("0° - 30°: .......... mRAD");
		JPanel gmpanel = new JPanel();
		gmpanel.setLayout(new BoxLayout(gmpanel, BoxLayout.Y_AXIS));
		gmpanel.setBorder(new CompoundBorder(new EtchedBorder(),new EmptyBorder(3,5,3,5)));
		gmpanel.add(gm);
		gmpanel.add(fsc);
		gmpanel.add(dynstab);
		gmpanel.setPreferredSize(new Dimension(175,65));
		gmpanel.setMinimumSize(new Dimension(175,65));
		gmpanel.setMaximumSize(new Dimension(175,65));
		
		//Stability info
		JPanel infopanel = new JPanel();
		infopanel.setLayout(new BoxLayout(infopanel, BoxLayout.Y_AXIS));
		JLabel gzcurve = new JLabel("Aproximate GZ-curve");
		gzcurve.setFont(new Font("Dialog",Font.BOLD,14));
		JLabel wallsided = new JLabel("(Calculated using the");
		JLabel wallsided2 = new JLabel("Wall-sided formula)");
		wallsided.setFont(new Font("Dialog",Font.PLAIN,10));
		wallsided2.setFont(new Font("Dialog",Font.PLAIN,10));
		infopanel.add(gzcurve);
		infopanel.add(wallsided);
		infopanel.add(wallsided2);
		infopanel.add(Box.createVerticalStrut(15));
		stabilityoutput.add(infopanel);
		stabilityoutput.add(gmpanel);
		
		//Stability Panel (output+ info)
		JPanel stability = new JPanel();
		stability.setLayout(new BorderLayout());
		stability.setBorder(new EmptyBorder(3,5,3,5));
		stability.add(BorderLayout.WEST,gzmainpanel);
		stability.add(BorderLayout.CENTER,stabilityoutput);
		
		//Ship
		shipdrawing = new DrawShip();
		shiprotated = new RotatePanel();
		shiprotated.add(shipdrawing);
		JPanel namepanel = new JPanel();
		JLabel name = new JLabel("M/T Danita");
		name.setFont(new Font("Dialog",Font.PLAIN,20));
		namepanel.add(name);
		JPanel nameandship = new JPanel();
		nameandship.setLayout(new BorderLayout());
		nameandship.add(BorderLayout.NORTH,namepanel);
		nameandship.add(BorderLayout.CENTER, shiprotated);
		
		//General condition
		JPanel generallabels = new JPanel();
		generallabels.setLayout(new BoxLayout(generallabels,BoxLayout.X_AXIS));
		generallabels.setBorder(new CompoundBorder(new EtchedBorder(),new EmptyBorder(3,5,3,5)));
		JPanel displacementpanel = new JPanel();
		displacementpanel.setLayout(new BoxLayout(displacementpanel, BoxLayout.Y_AXIS));
		displacementpanel.setBorder(new EmptyBorder(3,5,3,5));
		displacementpanel.setPreferredSize(new Dimension(190,122));
		displacementpanel.setMinimumSize(new Dimension(190,122));
		displacementpanel.setMaximumSize(new Dimension(190,122));
		JPanel trimpanel = new JPanel();
		trimpanel.setLayout(new BoxLayout(trimpanel, BoxLayout.Y_AXIS));
		trimpanel.setBorder(new EmptyBorder(3,5,3,5));
		trimpanel.setPreferredSize(new Dimension(190,122));
		trimpanel.setMinimumSize(new Dimension(190,122));
		trimpanel.setMaximumSize(new Dimension(190,122));
		JPanel tankpanel = new JPanel();
		tankpanel.setLayout(new BoxLayout(tankpanel, BoxLayout.Y_AXIS));
		tankpanel.setBorder(new EmptyBorder(3,5,3,5));
		tankpanel.setPreferredSize(new Dimension(190,122));
		tankpanel.setMinimumSize(new Dimension(190,122));
		tankpanel.setMaximumSize(new Dimension(190,122));
		JPanel stabilitypanel = new JPanel();
		stabilitypanel.setLayout(new BoxLayout(stabilitypanel, BoxLayout.Y_AXIS));
		stabilitypanel.setBorder(new EmptyBorder(3,5,3,5));
		stabilitypanel.setPreferredSize(new Dimension(190,122));
		stabilitypanel.setMinimumSize(new Dimension(190,122));
		stabilitypanel.setMaximumSize(new Dimension(190,122));
		JLabel displacementtitle = new JLabel("CONDITION  (ton)");
		displacementtitle.setFont(new Font("Default",Font.BOLD,12));
		displacement = new JLabel("Displacement: ");
		displacement.setForeground(Color.red);
		deadweight = new JLabel("Deadweight: ");
		deadweight.setForeground(Color.red);
		JLabel cargovolumetitle = new JLabel("<html>CARGO VOLUME (m<sup>3</sup>)</html>");
		cargovolumetitle.setFont(new Font("Default",Font.BOLD,12));
		cargovolume = new JLabel("Total volume: ");
		cargovolume.setForeground(Color.red);
		JLabel trimtitle = new JLabel("DRAUGHT  (m)");
		trimtitle.setFont(new Font("Default",Font.BOLD,12));
		besttrim = new JLabel("Trim:");
		draughtfwd = new JLabel("Fwd:");
		draughtaft = new JLabel("Aft:");
		tmd = new JLabel("TMD:");
		JLabel tanktitle = new JLabel("DISTRIBUTION  (ton)");
		tanktitle.setFont(new Font("Default",Font.BOLD,12));
		tank1 = new JLabel("Tank 1 P+S:");
		tank1.setForeground(new Color(48,217,72));
		tank2 = new JLabel("Tank 2 P+S:");
		tank2.setForeground(new Color(7,104,75));
		tank3 = new JLabel("Tank 3 P+S:");
		tank3.setForeground(new Color(29,187,144));
		tank4 = new JLabel("Tank 4 P+S:");
		tank4.setForeground(new Color(36,118,166));
		tank5 = new JLabel("Tank 5 P+S:");
		tank5.setForeground(new Color(6,207,225));
		tank6 = new JLabel("Tank 6 P+S:");
		tank6.setForeground(new Color(71,53,192));
		JLabel stabilitytitle = new JLabel("STABILITY  (m)");
		stabilitytitle.setFont(new Font("Default",Font.BOLD,12));
		gm2 = new JLabel("GM:");
		fsc2 = new JLabel ("FSC:");
		displacementpanel.add(displacementtitle);
		displacementpanel.add(Box.createVerticalStrut(5));
		displacementpanel.add(displacement);
		displacementpanel.add(deadweight);
		displacementpanel.add(Box.createVerticalStrut(10));
		displacementpanel.add(cargovolumetitle);
		displacementpanel.add(Box.createVerticalStrut(5));
		displacementpanel.add(cargovolume);
		trimpanel.add(trimtitle);
		trimpanel.add(Box.createVerticalStrut(5));
		trimpanel.add(draughtfwd);
		trimpanel.add(draughtaft);
		trimpanel.add(besttrim);
		trimpanel.add(tmd);
		tankpanel.add(tanktitle);
		tankpanel.add(Box.createVerticalStrut(5));
		tankpanel.add(tank1);
		tankpanel.add(tank2);
		tankpanel.add(tank3);
		tankpanel.add(tank4);
		tankpanel.add(tank5);
		tankpanel.add(tank6);
		stabilitypanel.add(stabilitytitle);
		stabilitypanel.add(Box.createVerticalStrut(5));
		stabilitypanel.add(gm2);
		stabilitypanel.add(fsc2);
		generallabels.add(displacementpanel);
		generallabels.add(new JSeparator(SwingConstants.VERTICAL));
		generallabels.add(trimpanel);
		generallabels.add(new JSeparator(SwingConstants.VERTICAL));
		generallabels.add(tankpanel);
		generallabels.add(new JSeparator(SwingConstants.VERTICAL));
		generallabels.add(stabilitypanel);
		
		//General condition
		JPanel generalcondition = new JPanel();
		generalcondition.setLayout(new BoxLayout(generalcondition,BoxLayout.Y_AXIS));
		generalcondition.setBorder(new EmptyBorder(3,5,3,5));
		generalcondition.add(nameandship);
		generalcondition.add(Box.createVerticalStrut(10));
		generalcondition.add(generallabels);
		
		//Genetic Output
		JPanel geneticoutput = new JPanel();
		geneticoutput.setBorder(new CompoundBorder(new EtchedBorder(),new EmptyBorder(3,5,3,5)));
		geneticoutput.setLayout(new GridLayout(1,4));
		Dimension d = new Dimension(150,20);
		generation = new JLabel("Generation: 0");
		generation.setMinimumSize(d);
		generation.setPreferredSize(d);
		generation.setMaximumSize(d);
		JPanel generationpanel = new JPanel();
		generationpanel.setBorder(new EmptyBorder(3,5,3,5));
		generationpanel.add(generation);
		bestfitness = new JLabel("Best fitness:");
		bestfitness.setMinimumSize(d);
		bestfitness.setPreferredSize(d);
		bestfitness.setMaximumSize(d);
		JPanel bestfitnesspanel = new JPanel();
		bestfitnesspanel.setBorder(new EmptyBorder(3,5,3,5));
		bestfitnesspanel.add(bestfitness);
		meanfitness = new JLabel("Mean fitness:");
		meanfitness.setMinimumSize(d);
		meanfitness.setPreferredSize(d);
		meanfitness.setMaximumSize(d);
		JPanel meanfitnesspanel = new JPanel();
		meanfitnesspanel.setBorder(new EmptyBorder(3,5,3,5));
		meanfitnesspanel.add(meanfitness);
		geneticoutput.add(generationpanel);
		geneticoutput.add(bestfitnesspanel);
		geneticoutput.add(meanfitnesspanel);
		
		//Genetic Output + Log
		JPanel genetic = new JPanel();
		genetic.setLayout(new BorderLayout());
		log	= new JTextArea(14, 58);
		log.setBorder(new EmptyBorder(3,5,3,5));
		log.setEditable(false);
		JScrollPane scroll = new JScrollPane(log);
		scroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		genetic.add(BorderLayout.NORTH, geneticoutput);
		genetic.add(BorderLayout.CENTER, scroll);
		
		//Strength curves
		sfdrawing = new DrawSF();
		bmdrawing = new DrawBM();
		JPanel sfandbmpanel = new JPanel();
		sfandbmpanel.setBorder(new EtchedBorder());
		sfandbm = new JLayeredPane();
		sfandbm.setPreferredSize(new Dimension(599, 300));
		sfandbm.add(sfdrawing,0);
		sfandbm.add(bmdrawing,1);
		sfandbmpanel.add(sfandbm);
		StrengthCurvesInfo legenda = new StrengthCurvesInfo();
		JPanel choosepanel = new JPanel();
		choosepanel.setLayout(new BoxLayout(choosepanel, BoxLayout.Y_AXIS));
		String[] sforbmlist = {"Shear Forces", "Bending Moments"};
		JComboBox choosesforbm = new JComboBox(sforbmlist);
		choosesforbm.setToolTipText("Choose the displayed curve");
		choosesforbm.addActionListener(new ComboBoxListener());
		JLabel approximation = new JLabel("*aprroximated model");
		approximation.setFont(new Font("Dialog",Font.PLAIN,9));
		choosepanel.add(legenda);
		choosepanel.add(choosesforbm);
		choosepanel.add(approximation);
		choosepanel.add(Box.createVerticalStrut(15));
		JLabel maxsflabel = new JLabel ("Max shear force (kton)");
		maxsf = new JLabel(" ");
		JLabel maxbmlabel = new JLabel ("Max bending moment (kton.m)");
		maxbm = new JLabel(" ");
		JPanel maxsfandbm = new JPanel();
		maxsfandbm.setLayout(new BoxLayout(maxsfandbm, BoxLayout.Y_AXIS));
		maxsfandbm.setBorder(new CompoundBorder(new EtchedBorder(),new EmptyBorder(3,5,3,5)));
		maxsfandbm.add(maxsflabel);
		maxsfandbm.add(maxsf);
		maxsfandbm.add(Box.createVerticalStrut(5));
		maxsfandbm.add(maxbmlabel);
		maxsfandbm.add(maxbm);
		JPanel sfandbminfo = new JPanel();
		sfandbminfo.setLayout(new BorderLayout());
		sfandbminfo.setBorder(new EmptyBorder(3,5,3,5));
		sfandbminfo.add(BorderLayout.NORTH, choosepanel);
		sfandbminfo.add(BorderLayout.CENTER, maxsfandbm);
		sfandbminfo.add(BorderLayout.SOUTH, Box.createVerticalStrut(90));
		JPanel strengthpanel = new JPanel();
		strengthpanel.setLayout(new BorderLayout());
		strengthpanel.add(BorderLayout.WEST, sfandbmpanel);
		strengthpanel.add(BorderLayout.CENTER, sfandbminfo);
		
		//Output tabbel pannel
		JTabbedPane output = new JTabbedPane();
		output.addTab("General condition", new ImageIcon(this.getClass().getResource("/images/table.png")), generalcondition,"Displays the general condition");
		output.addTab("Stability", new ImageIcon(this.getClass().getResource("/images/chart.png")), stability,"Displays stability information");
		output.addTab("Strength curves", new ImageIcon(this.getClass().getResource("/images/chart.png")), strengthpanel,"Displays shear forces and bending moments");
		output.addTab("Genetic", new ImageIcon(this.getClass().getResource("/images/log.png")), genetic, "Displays the genetic output");
		output.setBorder(new EmptyBorder(5,5,5,5));
		
		//Menu
		JMenuBar menuBar = new JMenuBar();
		JMenu file = new JMenu("File");
		file.setMnemonic(KeyEvent.VK_F);
		JMenuItem load = new JMenuItem("Load condition");
		//load.setEnabled(false);
		load.addActionListener(new OpenListener());
		load.setMnemonic(KeyEvent.VK_L);
		JMenuItem save = new JMenuItem("Save condition");
		//save.setEnabled(false);
		save.addActionListener(new SaveListener());
		save.setMnemonic(KeyEvent.VK_S);
		JMenuItem settings = new JMenuItem("Settings");
		settings.addActionListener(new SettingsListener());
		settings.setMnemonic(KeyEvent.VK_S);
		JMenuItem quit = new JMenuItem("Quit");
		quit.addActionListener(new QuitListener());
		quit.setMnemonic(KeyEvent.VK_Q);
		file.add(load);
		file.add(save);
		file.addSeparator();
		file.add(settings);
		file.addSeparator();
		file.add(quit);
		JMenu help = new JMenu("Info");
		help.setMnemonic(KeyEvent.VK_I);
		JMenuItem helpitem = new JMenuItem("Help");
		helpitem.addActionListener(new HelpListener());
		helpitem.setMnemonic(KeyEvent.VK_H);
		JMenuItem about = new JMenuItem("About");
		about.addActionListener(new AboutListener());
		about.setMnemonic(KeyEvent.VK_A);
		help.add(helpitem);
		help.addSeparator();
		help.add(about);
		menuBar.add(file);
		menuBar.add(help);
		
		
		// Frame
		frame = new JFrame("OptiLoading");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.getContentPane().add(BorderLayout.NORTH,inputandmisc);
		frame.getContentPane().add(BorderLayout.CENTER,output);
		frame.setIconImage(new ImageIcon(this.getClass().getResource("/images/icon.png")).getImage());
		frame.setJMenuBar(menuBar);
		frame.setResizable(false);
		frame.pack();
		frame.setLocationRelativeTo(null);
		frame.setVisible(true);
	}
	
	/**
	 * Method uses by the Applet version to show the main frame.
	 */
	public void showFrame(){
		frame.setVisible(true);
	}
	
	/**
	 * Method to add text to the genetic log.
	 * @param add String containing the text you want to add to the log.
	 */
	public void addLog(String add){
		log.append(add + "\n");
		log.setCaretPosition(log.getDocument().getLength());
	}
	
	/**
	 * Method to add a seperator to the log
	 */
	public void addLogSeparator(){
		for(int i = 0; i < 250; i++){
			log.append("-");
		}
		log.append("\n");
		log.setCaretPosition(log.getDocument().getLength());
	}
	
	//Listens to input fiels and checks if the values are acceptable
	class FieldListener implements FocusListener {
		public void focusLost(FocusEvent e) {
			updateLabels();
		}
		public void focusGained(FocusEvent e){}
	}
	
	//ActionListener to see if the shear forces or bending moment diagram has to be displayed
	class ComboBoxListener implements ActionListener {
        public void actionPerformed(ActionEvent e){
            JComboBox cb = (JComboBox)e.getSource();
            if (cb.getSelectedIndex() == 1){
				sfandbm.moveToFront(bmdrawing);
			} else{
				sfandbm.moveToFront(sfdrawing);
			}
        }
    }
	
	//ActionListener which is activated when the about button is pressed
    class AboutListener implements ActionListener{
		public void actionPerformed(ActionEvent e){
			new AboutBox();
		}
	}
	
	//ActionListener which is activated when the help button is pressed
	class HelpListener implements ActionListener{
		public void actionPerformed(ActionEvent e){
			new HelpViewer();
		}
	}

	//ActionListener which is activated when the quit button is pressed
	class QuitListener implements ActionListener{
		public void actionPerformed(ActionEvent e){
			System.exit(0);
		}
	}
	
	//ActionListener which is activated when the settings button is pressed
	class SettingsListener implements ActionListener{
		public void actionPerformed(ActionEvent e){
			//Check that the algorithm is not running before displaying the panel
			if(start.getText()=="Start"){
				Settings settingspanel = Settings.getInstance();
				settingspanel.setVisible(true);
				settingspanel = null;
			}else{
				JOptionPane.showMessageDialog(frame,"Please stop the simulation to change the settings.");
			}
		}
	}
	
	// ActionListener that listens to the StartButton
	class StartButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent e){
			JButton source = (JButton) e.getSource();
			if (source.getText()=="Start"){
				if(setCargo()){
				source.setText("Stop");
				
				//Disable all components
				inputcargo.setEnabled(false);
				specgrav.setEnabled(false);
				waterdens.setEnabled(false);
				inputtrim.setEnabled(false);
				tmdorcargo.setEnabled(false);
				misccompartments.isEnabled(false);
				
				//Hide the settings panel
				Settings settingspanel = Settings.getInstance();
				settingspanel.setVisible(false);
				settingspanel = null;
				
				//Add info to log
				addLog("Cargo Values set");
				addLog("Simulation started");
				addLogSeparator();
				
				// Start the genetics thread
				Genetics.start(); 
				} else	{JOptionPane.showMessageDialog(frame,"Please check input values.");}
			} else {
				Genetics.stop(); // Stop the genetics thread
				addLog("Simulation stopped");
				addLogSeparator();
				
				//Enable all components
				inputcargo.setEnabled(true);
				specgrav.setEnabled(true);
				waterdens.setEnabled(true);
				inputtrim.setEnabled(true);
				tmdorcargo.setEnabled(true);
				misccompartments.isEnabled(true);
				source.setText("Start");
			} 
		}
	}

	// ActionListener that will listen to the save button
	class SaveListener implements ActionListener{
		private JFileChooser fc = new JFileChooser();
		
		public void actionPerformed(ActionEvent e){
			if((start.getText()=="Start") &&
			  (inputcargo.getValue() != null) &&
			  (inputtrim.getValue()!= null) &&
			  (bestchrom != null)){
				fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
				fc.setMultiSelectionEnabled(false);
				int returnVal = fc.showSaveDialog(frame);
				if (returnVal == JFileChooser.APPROVE_OPTION){
					File file = fc.getSelectedFile();
					FileData filedata = new FileData();
					try{
						FileOutputStream fs = new FileOutputStream(file);
						ObjectOutputStream os = new ObjectOutputStream(fs);
						os.writeObject(filedata);
						os.close();
					} catch (Exception ex){
						JOptionPane.showMessageDialog(frame,"Error while saving file.");
						ex.printStackTrace();
					}
				}
			} else{
				JOptionPane.showMessageDialog(frame,"Not ready to save. Please ensure that the simulation\n has been stopped and a condition has been found.");
			}
		}
	}
	
	//ActionListener that will listen to the open button
	class OpenListener implements ActionListener{
		private JFileChooser fc = new JFileChooser();

		public void actionPerformed(ActionEvent e){
			if(start.getText()=="Start"){
				fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
				fc.setMultiSelectionEnabled(false);
				int returnVal = fc.showOpenDialog(frame);
				FileData filedata = null;
				if (returnVal == JFileChooser.APPROVE_OPTION){
					File file = fc.getSelectedFile();
					try{
						FileInputStream fs = new FileInputStream(file);
						ObjectInputStream is = new ObjectInputStream(fs);
						filedata = (FileData) is.readObject();
						is.close();
					} catch (Exception ex){
						JOptionPane.showMessageDialog(frame,"Error loading file.");
						ex.printStackTrace();
					}
				}
				//Show data in gui.
				repaintOptiloading(0,0,filedata.best,filedata.usedShip);
				setConstants(filedata.usedShip.getData(ShipParticular.DISPLACEMENT),
							 filedata.usedShip.getData(ShipParticular.DEADWEIGHT),
							 filedata.usedShip.getData(ShipParticular.DRAUGHT));
				misccompartments.importRowlist(filedata.rowdata);
				inputcargo.setValue(filedata.tmdorcargo);
				specgravfield.setValue(filedata.specificgravity);
				inputtrim.setValue(filedata.targettrim);
				waterdensfield.setValue(filedata.waterdens);
				NumberFormat n = NumberFormat.getInstance();
				n.setMaximumFractionDigits(2);
				cargovolume.setText("Total volume: "+ n.format(filedata.specificgravity*filedata.tmdorcargo));
				cargovolume.setForeground(Color.black);
			} else {
				JOptionPane.showMessageDialog(frame,"Please stop the simulation to load a condition.");
			}
			
		}
	}

	/**
	 * Method to get the desired cargo or TMD to be achieved.
	 * @return Floating point value containing the cargo or TMD to be achieved.
	 */
	public float getCargo(){
		Number f = (Number) inputcargo.getValue();
		return  f.floatValue();
	}
	
	/**
	 * Method to get update the displacement and deadweight label.
	 * @param dis Floating point value that holds the displacement.
	 * @param dead Floating point value that holds the deadweight.
	 */
	public void setConstants(float dis, float dead,float tmdvalue){
		// To set the fraction digits and format the numbers
		NumberFormat n = NumberFormat.getInstance();
		n.setMaximumFractionDigits(2);
		displacement.setText("Displacement: "+ n.format(dis));
		displacement.setForeground(Color.black);
		deadweight.setText("Deadweight: "+n.format(dead));
		deadweight.setForeground(Color.black);
		tmd.setText("TMD: "+n.format(tmdvalue));
	}
	
	/**
	 * Method to get the specific gravity of the cargo.
	 * @return Floating point value containing the specific gravity of the cargo.
	 */
	public float getSpecificgravity(){
		Number f = (Number) specgravfield.getValue();
		return  f.floatValue();
	}
	
	/**
	 * Method to get the density of the water.
	 * @return Floating point value containing the density of the water.
	 */
	public float getDensity(){
		Number f = (Number) waterdensfield.getValue();
		return  f.floatValue();
	}
	
	/**
	 * Method to get the desired trim.
	 * @return Floating point value containing the desired trim.
	 */
	public float getTarget(){
		Number f = (Number) inputtrim.getValue();
		return  f.floatValue();
	}
	
	/**
	 * Method which is calles when an optimal condition has been found
	 */
	public void stopFinal(){
		start.doClick();
		JOptionPane.showMessageDialog(frame,"Optimal solution found!");
	}
	
	/**
	 * Method to get the best chromosome.
	 * @return Chromosome that holds the best solution.
	 */
	public Chromosome getBestchrom(){
		return bestchrom;
	}
	
	// Method to set the cargo and check if the input is correct
	private boolean setCargo(){
		boolean returnvalue = false;
		Ship loadedship = Ship.getInstance();
		
		// Check to see that the input values have been filled and the target is an acceptable value
		if ((inputcargo.getValue() != null) &&
		    (inputtrim.getValue()!= null) &&
			(getTarget()<=5) && (getTarget() >= -5)){
			
			// case statement to switch between a desired tmd or cargo value
			switch(tmdorcargo.getSelectedIndex()){
				case 0:
				// cargo value
				Cargo inputcargo = new Cargo(getCargo(),getSpecificgravity()); //set cargo
				if (loadedship.checkToLoad(inputcargo, getDensity(), getTarget(),misccompartments.getConstants())){ //check cargo
					returnvalue = true;
				}
				break;
				
				case 1:
				// tmd value
				if ((getCargo()>= 4) && (getCargo() <= 16.9f)){
					if (loadedship.checkToLoad(getCargo(),getSpecificgravity(),getDensity(), getTarget(),misccompartments.getConstants())){ //check tmd
						returnvalue = true;
					}
				}
				break;
			}
		
		}
		return returnvalue;
	}

	/**
	 * Method to repaint the main GUI, if best solution hasn't changed.
	 * @param gen Integer value containing the current generation.
	 * @param mean Floating point value containing the mean fitness of the current population.
	 */
	public void repaintOptiloading(int gen, float mean){
		NumberFormat n = NumberFormat.getInstance();
		n.setMaximumFractionDigits(2);
		generation.setText("Generation: "+String.valueOf(gen));
		meanfitness.setText("Mean fitness: "+ n.format(mean));
	}
	
	/**
	 * Method to repaint the main GUI.
	 * @param gen Integer value containing the current generation.
	 * @param best Floating point value containing the fitness of the best solution.
	 * @param mean Floating point value containing the mean fitness of the current population.
	 * @param best Chromosome object that is the best of its current population.
	 * @param loadedship Ship object which contains general ship information.
	 */
	public void repaintOptiloading(int gen, float mean, Chromosome best, Ship loadedship){
		float specgrav = loadedship.toLoad.getSpecificgravity();
		bestchrom = best;
		
		//Shear forces and bending moments
		ArrayList<Intensity> newintensitylist = new ArrayList<Intensity>();
		
		//Get a clone of the ship's intensitylist
		for(Intensity x: loadedship.intensitylist){
			newintensitylist.add((Intensity) x.clone());
		}
		
		//Add the intensity value of each tank
		int j=0;
		for(Tank x: loadedship.tanks){
			float p = (float) ((best.getVolumes().get(j)*specgrav)/(-x.getLength()));
			for(int i = (x.getBeginframe()+1); i < (x.getEndframe()+1); i++){
				newintensitylist.get(i).addIntensity(p);
			}
			j++;
		}
		
		//Initialize the values
		float sf = 0;
		float tempsf;
		float bm = 0;
		float tempbm;
		float maxshearforce = 0;
		float sfframe = 0;
		float maxbendingmoment = 0;
		float bmframe = 0;
		int sfx;
		int sfy;
		int bmy;
		float dist;
		sfdrawing.newList();
		bmdrawing.newList();
		
		//Draw the shear force and bending moment for each frame
		for(int i = 1; i < newintensitylist.size();i++){
			dist = newintensitylist.get(i).getDistfromap()-newintensitylist.get(i-1).getDistfromap();
			tempsf = newintensitylist.get(i).getIntensity()*dist;
			sf += (tempsf/3);
			sfx = (int) (Math.round((newintensitylist.get(i).getDistfromap()*547)/218.7) + 37);
			sfy = (int) (150 - Math.round((sf*150)/21000));
			sfdrawing.addList(sfx,sfy);
			tempbm = (float) (0.5*(sf+sf-tempsf)*dist);
			bm += tempbm;
			bmy = (int) (150 - Math.round((bm*150)/300000));
			bmdrawing.addList(sfx,bmy);
			if(Math.abs(sf)>Math.abs(maxshearforce)){
				maxshearforce = sf;
				sfframe = i;
			}
			if(Math.abs(bm)>Math.abs(maxbendingmoment)){
				maxbendingmoment = bm;
				bmframe = i;
			}
		}
		
		bmdrawing.bmCorrection();
		
		// To set the fraction digits and format the numbers
		NumberFormat n = NumberFormat.getInstance();
		NumberFormat m = NumberFormat.getInstance();
		n.setMaximumFractionDigits(3);
		m.setMaximumFractionDigits(0);
		maxsf.setText(n.format(maxshearforce/1000)+" @ "+m.format(sfframe));
		maxbm.setText(n.format(maxbendingmoment/1000)+" @ "+m.format(bmframe));
		
		// Trim and draught
		besttrim.setText("Trim: "+n.format(best.getTrim()));
		n.setMaximumFractionDigits(2);
		draughtaft.setText("Aft: "+n.format(best.getDaft()));
		draughtfwd.setText("Fwd: "+n.format(best.getDfwd()));
		
		// Set parameters to draw ship
		shipdrawing.setVolumes(best.getVolumes());
		shiprotated.setTrim(best.getTrim());
		
		// Tank labels
		n.setMaximumFractionDigits(1);
		tank1.setText("Tank 1 P+S: "+n.format(best.getVolumes().get(0) * specgrav));
		tank2.setText("Tank 2 P+S: "+n.format(best.getVolumes().get(1) * specgrav));
		tank3.setText("Tank 3 P+S: "+n.format(best.getVolumes().get(2) * specgrav));
		tank4.setText("Tank 4 P+S: "+n.format(best.getVolumes().get(3) * specgrav));
		tank5.setText("Tank 5 P+S: "+n.format(best.getVolumes().get(4) * specgrav));
		tank6.setText("Tank 6 P+S: "+n.format(best.getVolumes().get(5) * specgrav));
		
		// Genetic output
		n.setMaximumFractionDigits(2);
		generation.setText("Generation: "+String.valueOf(gen));
		bestfitness.setText("Best fitness: "+n.format(best.getFitness()));
		meanfitness.setText("Mean fitness: "+ n.format(mean));
		
		//Stability output
		gm.setText("GM: "+n.format(best.getGM()) + " m");
		fsc.setText("FSC: "+n.format(best.getFSC()) + " m");
		gm2.setText("GM: "+n.format(best.getGM()));
		fsc2.setText("FSC: "+n.format(best.getFSC()));
		n.setMaximumFractionDigits(4);
		dynstab.setText("0° - 30°: "+n.format(best.getDynstab()) + " mRAD");
		gzpanel.setValues(best.getGM(),best.getBM());
		
		frame.repaint(); //render the ship + tank levels
		
		loadedship = null;
	}
	
	//Inner class to draw the legenda for the strength curves
	private class StrengthCurvesInfo extends JPanel{
		
		//Constructor
		StrengthCurvesInfo(){
			setPreferredSize(new Dimension(180,80));
			setMinimumSize(new Dimension(180,80));
			setMaximumSize(new Dimension(180,80));
			setSize(180,80);
		}

		//Paint the legenda
		public void paintComponent(Graphics g){
			Graphics2D g2d = (Graphics2D)g;
			g2d.setFont(new Font("Dialog",Font.BOLD,12));
			
			//Real curve
			g2d.setColor(Color.gray.brighter());
			g2d.drawString("Real", 32,22);
			g2d.setColor(Color.white);
			Line2D line = new Line2D.Float(5,15,25,15);
			drawShadow(line,g2d);
			g2d.draw(line);
			g2d.drawString("Real", 30,20);

			//Still water limit
			g2d.setColor(Color.gray.brighter());
			g2d.drawString("Still water limit", 32,42);
			g2d.setColor(Color.yellow);
			line = new Line2D.Float(5,35,25,35);
			drawShadow(line,g2d);
			g2d.draw(line);
			g2d.drawString("Still water limit", 30,40);

			//Wave limit
			g2d.setColor(Color.gray.brighter());
			g2d.drawString("Wave limit", 32,62);
			g2d.setColor(Color.red);
			g2d.drawString("Wave limit", 30,60);
			line = new Line2D.Float(5,55,25,55);
			drawShadow(line,g2d);
			g2d.draw(line);
		}
		
		//Method to draw the shadow
		private void drawShadow(Line2D line, Graphics2D g2)
	    {
	        Paint orig = g2.getPaint();
	        AffineTransform at = AffineTransform.getTranslateInstance(3.0,2.0);
	        Shape shadow = at.createTransformedShape(line);
	        g2.setPaint(Color.gray.brighter());
	        g2.draw(shadow);
	        g2.setPaint(orig);
	    }
	}
	
	/**
	  * Method to show if the input values are acceptable.
	  */
	public void updateLabels(){
		float cargoweight;
		float constants = misccompartments.getConstants();
		float waterdensity = getDensity();
		float cargodens = getSpecificgravity();
		Ship usedShip = Ship.getInstance();
		NumberFormat n = NumberFormat.getInstance();
		n.setMaximumFractionDigits(2);
		try {
			inputcargo.commitEdit();
			specgrav.commitEdit();
			waterdens.commitEdit();
		} catch (ParseException e) {}
		
		if(inputcargo.getValue() != null){
			if(tmdorcargo.getSelectedIndex()== 0){
				cargoweight = getCargo();
			}else{
				cargoweight = usedShip.getDispAtTMD(getCargo(), waterdensity)-constants-14910;
			}
			if(usedShip.checkDeadweight(constants+cargoweight,waterdensity)){
				displacement.setForeground(Color.black);
				displacement.setText("Displacement: "+ n.format(constants+cargoweight+14910));
				deadweight.setForeground(Color.black);
				deadweight.setText("Deadweight: "+n.format(constants+cargoweight));
			}else{
				displacement.setForeground(Color.red);
				displacement.setText("Displacement: "+ n.format(constants+cargoweight+14910));
				deadweight.setForeground(Color.red);
				deadweight.setText("Deadweight: "+n.format(constants+cargoweight));
			}
			if(usedShip.checkVolume(cargoweight/cargodens)){
				cargovolume.setForeground(Color.black);
				cargovolume.setText("Total volume: "+n.format(cargoweight/cargodens));
			} else{
				cargovolume.setForeground(Color.red);
				cargovolume.setText("Total volume: "+n.format(cargoweight/cargodens));
			}
		} else {
			if(usedShip.checkDeadweight(constants,waterdensity)){
				displacement.setForeground(Color.black);
				displacement.setText("Displacement: "+ n.format(constants+14910));
				deadweight.setForeground(Color.black);
				deadweight.setText("Deadweight: "+n.format(constants));
			}else{
				displacement.setForeground(Color.red);
				displacement.setText("Displacement: "+ n.format(constants+14910));
				deadweight.setForeground(Color.red);
				deadweight.setText("Deadweight: "+n.format(constants));
			}
		}			
		
	}
}