//: be/optiloading/Intensity.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading;

import java.io.*;

/** 
 * Intensity class used to calculate shear forces and bending moments.
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank" href="http://www.optiloading.be">www.optiloading.be</a>
 */

public class Intensity implements Cloneable , Serializable{
	private float p;
	private float distfromap;
	
	/**
	 * Constructor used to create a new intensity object.
	 * @param dis Floating point value indication the distance from the aft perpendicular.
	 * @param in Floating point value indication the intensity in t/m.
	 */
	public Intensity(float dis, float in){
		p = in;
		distfromap = dis;
	}
	
	/**
	 * Change the intensity of the object with this value.
	 * @param add foating point value indicating the intensity to add in t/m (can be negative).
	 */
	public void addIntensity(float add){
		p += add;
	}
	
	/**
	 * Get the intensity value of the current object.
	 * @return Floating point value indication the intensity of the current object.
	 */
	public float getIntensity(){
		return p;
	}
	
	/**
	 * Get the distance from the aft perpendicular of the current object.
	 * @return Floating point value indication the distance from the aft perpendicular of the current object.
	 */
	public float getDistfromap(){
		return distfromap;
	}
	
	/**
	 * Overwritten method to convert the object to a String.
	 * @return Sting with the values of the object.
	 */
	public String toString(){
		String returnvalue = String.valueOf(distfromap) + " & "+ String.valueOf(p);
		return returnvalue; 
	}
	
	/**
	 * Overwritten method to return a clone of the object.
	 * @return Object clone of the object (can be cast as Intensity).
	 */
	public Object clone(){
		try {
			Object clone = super.clone(); // Make clone
			return clone;
		} catch (CloneNotSupportedException e){
			throw new InternalError();
		}
	}
}