//: be/optiloading/Main.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import javax.imageio.*;
import java.io.IOException;
import javax.swing.border.*;


/** 
 * Controller class used to load the Ship and Gui object
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank" href="http://www.optiloading.be">http://www.optiloading.be</a>
 */

public class Main {
	private static Ship usedShip;
	private static Gui optiloading;
	
	/**
	 * Sole entry point to the class and application.
	 * @param args Array of String arguments.
	 * @exception java.lang.InterruptedException
	 *            Thrown from the Thread class (main thread).
	 */
	public static void main(String[] args) {
		//Create the window and the panel for the splash screen
		JWindow splash = new JWindow();
		JPanel content = (JPanel)splash.getContentPane();

		//Create a border
		content.add(BorderLayout.CENTER, new Splash());
		
		//Show the splash screen in the middle
		splash.setSize(332,208);
		splash.setLocationRelativeTo(null);
		splash.setVisible(true);
		
		//Load the program and pause for 2 seconds (to avoid a quick flash on fast machines)
		try{
		Thread.sleep(1500);
		} catch (Exception e) {}

		loadProgram();

		//Destroy the splash screen
		splash.setVisible(false);
		splash.dispose();
	}
	
	//Load the GUI and ship object while the splash screen is showing
	private static void loadProgram(){
		// Get an instance of the Ship class
		// This is the first time the getInstance() method is called so a new Ship object will be created
		usedShip = Ship.getInstance();
		
		// Get an instance of the Gui class
		// This is the first time the getInstance() method is called so a new Gui object will be created
		optiloading = Gui.getInstance();
		
		// Add information to log
		optiloading.addLog("Gui and ship loaded");
		optiloading.addLogSeparator();
	}
	
	//Class to draw the splash screen
	static class Splash extends JPanel{
		public Splash(){
			Dimension d5 = new Dimension(332,208);
			setPreferredSize(d5);
			setMinimumSize(d5);
			setMaximumSize(d5);
		}
		public void paintComponent(Graphics g){
			BufferedImage image = null;
			try {
				image = ImageIO.read(this.getClass().getResource("/images/splash.png"));
			} catch (IOException e) {
			}
			
			Graphics2D g2d = (Graphics2D)g;

			RenderingHints rh = new RenderingHints(
					RenderingHints.KEY_ANTIALIASING,
					RenderingHints.VALUE_ANTIALIAS_ON);
			g2d.setRenderingHints(rh);

			g2d.drawImage(image,0,0,null);
		}
	}
}