//: be/optiloading/MiscComp.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading;

import javax.swing.*;
import javax.swing.text.*;
import javax.swing.table.*;
import java.util.*;
import java.io.*;
import java.awt.*;
import java.text.*;
import java.awt.event.*;
import javax.swing.border.*;

/** 
 * MiscComp class is creates the table with the miscellaneous compartments.
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank" href="http://www.optiloading.be">www.optiloading.be</a>
 */

class MiscComp extends JPanel{
	public ArrayList<Tank> misccomp = new ArrayList<Tank>(); //ArrayList which holds all miscellaneous compartments
	private ArrayList<TableData> rowlist; //ArrayList which holds each row
	private JTable table;
	private static MiscComp uniqueInstance;
	
	public static MiscComp getInstance(){
		
		//Create a new Gui object when no Gui object is in existance.
		if (uniqueInstance == null){
			uniqueInstance = new MiscComp();
		}
		return uniqueInstance;
	}
	
	/**
	 * Constructor to create the GUI components and set the TableModel
	 */
	private MiscComp(){
		super(new GridLayout(1,0));
		setBorder(new CompoundBorder(
					 new TitledBorder(new EtchedBorder(),"Miscellaneous compartments"),
				 	 new EmptyBorder(5,5,0,5)));
		
		loadMiscData(this.getClass().getResourceAsStream("/data/misccomp.dat"));
		rowlist = new ArrayList<TableData>();
		for(Tank x: misccomp){
			rowlist.add(new TableData(x.getTankid(),0,0,0,0));
		}

        table = new JTable(new JTableModel());
        table.setPreferredScrollableViewportSize(new Dimension(495, 100));
		table.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
		JScrollPane scrollPane = new JScrollPane(table);
        
		//Set a spinner render for the % full column
		TableColumn col = table.getColumnModel().getColumn(3);
		col.setCellEditor(new SpinnerEditor());
		col.setCellRenderer(new PercentageRenderer());
		
		//Set number renderers for the vol, weight and density column
		NumberFormat numberDisplayFormat = NumberFormat.getInstance();
		numberDisplayFormat.setMinimumFractionDigits(2);
		
		col = table.getColumnModel().getColumn(1);
		col.setCellEditor(new NumberEditor());
		col.setCellRenderer(new NumberRenderer());
		
		col = table.getColumnModel().getColumn(2);
		col.setCellEditor(new NumberEditor());
		col.setCellRenderer(new NumberRenderer());
		
		col = table.getColumnModel().getColumn(4);
		col.setCellEditor(new NumberEditor());
		col.setCellRenderer(new NumberRenderer());
		
		// Give the compartment name column a perferred width
		col = table.getColumnModel().getColumn(0);
		col.setPreferredWidth(140);
		
		JTableHeader header = table.getTableHeader();
		Dimension dim = header.getPreferredSize();
		dim.height +=3;
		header.setPreferredSize(dim);
		
		add(scrollPane);
	}
	
	//Private method to load the data from each miscellaneous compartment
	private void loadMiscData(InputStream miscdata){
		try{
			BufferedReader reader = new BufferedReader(new InputStreamReader(miscdata)); //Get a reader to read the file
			Tank currentTank = null;
			String line = null;
			
			//Run trough the file and create a new compartment when the line starts with "N".
			//Stow the tankdata of each tank in the object.
			//Stow the Tank object in the public ArrayList<Tank> misccomp.
			while((line=reader.readLine()) != null){
				String[] row = line.split(";");
				if (row[0].equals("N")){
					currentTank = new Tank(row[1],Float.parseFloat(row[2]),Float.parseFloat(row[3]),Integer.parseInt(row[4]),Integer.parseInt(row[5]));
					misccomp.add(currentTank);
				} else {
					currentTank.addData(Float.parseFloat(row[0]), Float.parseFloat(row[1]), Float.parseFloat(row[2]), Float.parseFloat(row[3]));
				}
			}
			reader.close(); //Close the file
		} catch(Exception ex) {
			System.out.println("Cannot read misc file");
		}
	}

	//Inner class to create the spinner editor for the % full column
	private class SpinnerEditor extends AbstractCellEditor implements TableCellEditor {
		private JSpinner spinner;
		
	    // Initializes the spinner.
		public SpinnerEditor() {
			spinner = new JSpinner(new SpinnerNumberModel(0,0,1,.001));
			spinner.setEditor(new JSpinner.NumberEditor(spinner, "0.0%"));
			spinner.setBorder(null);
	    }

	    // Prepares the spinner component and returns it.
		public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
			spinner.setValue(value);
			return spinner;
		}

		// Enables the editor only for double-clicks.
		public boolean isCellEditable(EventObject evt) {
			return true;
	    }

		// Returns the spinners current value.
		public Object getCellEditorValue() {
			JFormattedTextField t = ((JSpinner.DefaultEditor) spinner.getEditor()).getTextField();
			String s = t.getText();
			if(s.indexOf("%")==-1){
				t.setText(s+"%");
			}
			
			try {spinner.commitEdit();
			} catch (ParseException e) { }
			
			return spinner.getValue();
	    }
	}
	
	//Inner class to create a percentage renderer for the % full column
	private class PercentageRenderer implements TableCellRenderer{
		private NumberFormat percentDisplayFormat;
		private JFormattedTextField percentage; 
		
		//Constructor      
		PercentageRenderer(){
			percentDisplayFormat = NumberFormat.getPercentInstance();
			percentDisplayFormat.setMinimumFractionDigits(2);
			percentage = new JFormattedTextField(
			                             new DefaultFormatterFactory(
			                                new NumberFormatter(percentDisplayFormat),
			                                new NumberFormatter(percentDisplayFormat)));
			percentage.setBorder(null);
			percentage.setHorizontalAlignment(JTextField.RIGHT);
		}
		
		//Overwritten method to get the renderer
		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column){
			percentage.setValue(value);
			return percentage;
		}
	}
	
	//Inner class to create a number editor
	private class NumberEditor extends AbstractCellEditor implements TableCellEditor {
		private JFormattedTextField formattedtextfield;
		
		//Constructor
		public NumberEditor() {
			NumberFormat  numberDisplayFormat = NumberFormat.getInstance();
			numberDisplayFormat.setMinimumFractionDigits(2);
			formattedtextfield = new JFormattedTextField(numberDisplayFormat);
			formattedtextfield.setHorizontalAlignment(JTextField.RIGHT);
			formattedtextfield.setBorder(null);
	    }

		//Overwritten method to get the editor
		public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
			formattedtextfield.setValue(value);
			return formattedtextfield;
		}

		//Overwritten method to see if the cell is editable
		public boolean isCellEditable(EventObject evt) {
			return true;
	    }

		//Overwritten method to get the value of the cell
		public Object getCellEditorValue() {
			try {formattedtextfield.commitEdit();
			} catch (ParseException e) { /* do nothing */ }
			return formattedtextfield.getValue();
	    }
	}
	
	//Inner class to create a number renderer
	private class NumberRenderer extends DefaultTableCellRenderer{
		protected Format m_format;
		
		//Constructor
		NumberRenderer(){
			NumberFormat  numberDisplayFormat = NumberFormat.getInstance();
			numberDisplayFormat.setMinimumFractionDigits(2);
			m_format = numberDisplayFormat;
			setHorizontalAlignment(JTextField.RIGHT);
		}
		
		//Overwritten method to get the renderer
		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column){
			return super.getTableCellRendererComponent(table,
				value==null ? null : m_format.format(value),
				isSelected, hasFocus, row, column);
		}
	}
	
	//Inner class which defines the table model
	private class JTableModel extends AbstractTableModel {
		private String[] columnNames = {"Compartment","<html>Density (t/m<sup>3</sup>)</html>","<html>Volume (m<sup>3</sup>)</html>","% Full", "Weight (t)"};
		
		//Get the number of columns
		public int getColumnCount() {
			return columnNames.length;
		}
		
		//Get the number of rows
		public int getRowCount() {
			return rowlist.size();
		}

		//Get the header name of the column
		public String getColumnName(int col) {
			return columnNames[col];
		}

		//Get the value of the cell
		public Object getValueAt(int row, int col) {
			TableData x = rowlist.get(row);
			return x.get(col);
		}
		
		//Get the column class
		public Class getColumnClass(int c) {
            return getValueAt(0, c).getClass();
        }

		//Define which columns are editable (only column 0 is not editable)
		public boolean isCellEditable(int row, int col) {
            if (col < 1) {
                return false;
            } else {
                return true;
            }
        }

		//Set a value in a certain cell and adjust the other values
        public void setValueAt(Object value, int row, int col) {
			TableData x = rowlist.get(row);
			Number f = (Number) value;
			float flo =  f.floatValue();
			float maxvolume = misccomp.get(row).getMaxvolume();
			float dens = (Float) x.get(1);
			float volume = (Float) x.get(2);
            
			//Depending on which column changed, the other columns data is also adapted
			switch(col){
				case 1:
					x.set(1,flo);
					x.set(4,flo*volume);
					break;
				case 2:
					if(flo <= maxvolume){
						x.set(2,flo);
						x.set(3,flo/maxvolume);
						x.set(4,flo*dens);
					}
					break;
				case 3:
					x.set(2,flo*maxvolume);
					x.set(3,flo);
					x.set(4,flo*maxvolume*dens);
					break;
				case 4:
					if((dens !=0) && ((flo/dens)<= maxvolume)){
						x.set(2,flo/dens);
						x.set(3,(flo/dens)/maxvolume);
						x.set(4,flo);
					}
					break;		
			}
			fireTableDataChanged();
			Gui optiloading = Gui.getInstance();
			optiloading.updateLabels();
			optiloading = null;
        }
	}
	
	/**
	 * Get the sum of the weight of all miscellaneous compartments.
	 * @return Floating point value indication the total weight of all miscellaneous compartments.
	 */
	public float getConstants(){
		float sum = 0;
		for(TableData x: rowlist){
			sum += (Float) x.get(4);
		}
		return sum;
	}
	
	/**
	 * If the total deadweight is not exceeding the limit the miscelaneous tans are loaded.
	 */
	public void loadTanks(){
		for(int i = 0; i < misccomp.size(); ++i){
			Tank tank = misccomp.get(i);
			TableData data = rowlist.get(i);
			tank.setVolume((Float) data.get(2),(Float) data.get(1));
		}
	}
	
	/**
	 * Get the longitudinal, vertical and free surface moment created by the miscelaneous compartments.
	 * @return ArrayList containing floating point values indication the moments.
	 */
	public ArrayList<Float> getMoments(){
		ArrayList<Float> returnvalue = new ArrayList<Float>();
		float longmoment = 0;
		float vertmoment = 0;
		float fsm = 0;
		
		for(Tank x: misccomp){
			longmoment += (x.getCargo()*x.getLCG());
			vertmoment += (x.getCargo()*x.getKG());
			fsm += (x.getSpecificGravity()*x.getFSM());
		}
		
		returnvalue.add(longmoment);
		returnvalue.add(vertmoment);
		returnvalue.add(fsm);
		return returnvalue;
	}
	
	/**
	 * Get the intensity created by the miscellaneous compartments.
	 * @return ArrayList containing intensity values of all miscellaneous compartments.
	 */
	public ArrayList<Float> getIntensityList(){
		ArrayList<Float> returnvalue = new ArrayList<Float>();
		
		for(int i = 0; i < 249;i++){
			returnvalue.add(0f);
		}
		for(Tank x: misccomp){
			float p = x.getCargo()/x.getLength();
			for(int i = (x.getBeginframe()+1); i < (x.getEndframe()+1);i++){
				returnvalue.set(i,(returnvalue.get(i)-p));
			}
		}
		
		return returnvalue;
	}
	
	/**
	 * Method to disable the table (when the algorithm is running).
	 * @param b Boolean false is disabled, true is enabled.
	 */
	public void isEnabled(boolean b){
		table.setEnabled(b);
	}
	
	public ArrayList exportRowlist(){
		return rowlist;
	}
	
	public void importRowlist(ArrayList imported){
		rowlist = (ArrayList<TableData>) imported;
		JTableModel model = (JTableModel) table.getModel();
		model.fireTableDataChanged();
		
	}
}

//Class which holds one row of table data
class TableData implements Serializable{
	private String compartment;
	private float density;
	private float volume;
	private float percentage;
	private float weight;

	//Constructor
	TableData(String c, float d, float v,float p,float w){
		compartment = c;
		density= d;
		volume = v;
		percentage = p;
		weight = w;
	}

	//Get the value from a certain object
	public Object get(int i){
		Object returnvalue = null;
		switch(i){
			case 0: returnvalue = compartment; break;
			case 1: returnvalue = density; break;
			case 2: returnvalue = volume; break;
			case 3: returnvalue = percentage; break;
			case 4: returnvalue = weight; break;
		}
		return returnvalue;
	}
	
	//Set the value of a certain value
	public void set(int i,float f){
		switch(i){
			case 1: density = f; break;
			case 2: volume = f; break;
			case 3: percentage = f; break;
			case 4: weight= f; break;
		}
	}
}
