//: /be/optiloading/RotatePanel.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading;

import java.awt.*;
import javax.swing.*;
import java.math.*;

/** 
 * RotatePanel class which creates a rotating panel, which looks at the trim value of the ship to know how much to rotate.
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank" href="http://www.optiloading.be">www.optiloading.be</a>
 */

public class RotatePanel extends JPanel {
	
	private float trim = 0f;
	
	/**
	 * Overwritten paintComponent method to draw the panel under the appropriate angle.
	 * @param g Graphics object containing what to render.
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
        Graphics2D g2d = (Graphics2D) g;

		//RenderingHints to properly display the rotated ship
		RenderingHints rh = new RenderingHints(
				RenderingHints.KEY_INTERPOLATION,
				RenderingHints.VALUE_INTERPOLATION_BICUBIC);
		rh.put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		rh.put(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
		g2d.setRenderingHints(rh);
		// Rotate the panel to an angle four times greater then the angle of the ship in relation to the waterplane
		// Using the method rotate(angle, x value of the origin of rotation, y value of the origin of rotation)
		// angle = arcsin(-2*trim/Length/2)= arcsin(-4*trim/length)
		// x = 291 (middle value)
		// y = 54 (middle value)
        g2d.rotate(Math.asin(-(2*trim)/109.35) / 3, 291, 54);
     }
	
	/**
	 * Method to set the trim which have to be displayed.
	 * @param t Floating point value indicating the trim to be displayed.
	 */
	public void setTrim(float t){
		trim = t;
	}
}